﻿using ConfectioneryContracts.SearchModels;
using ConfectioneryContracts.StoragesContracts;
using ConfectioneryContracts.BindingModels;
using ConfectioneryContracts.BusinessLogicsContracts;
using ConfectioneryContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace ConfectioneryBusinessLogic.BusinessLogics
{
    public class MessageInfoLogic : IMessageInfoLogic
    {
        private readonly ILogger _logger;

        private readonly IMessageInfoStorage _messageInfoStorage;

        private readonly IClientStorage _clientStorage;

        public MessageInfoLogic(ILogger<MessageInfoLogic> logger, IMessageInfoStorage messageInfoStorage, IClientStorage clientStorage)
        {
            _logger = logger;
            _messageInfoStorage = messageInfoStorage;
            _clientStorage = clientStorage;
        }

        public List<MessageInfoViewModel>? ReadList(MessageInfoSearchModel? model)
        {
            _logger.LogInformation("ReadList. MessageId: {MessageId}. ClientId: {ClientId}", model?.MessageId, model?.ClientId);
            var list = model == null ? _messageInfoStorage.GetFullList() : _messageInfoStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }

        public bool Create(MessageInfoBindingModel model)
        {
            CheckModel(model);
            if (_messageInfoStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(MessageInfoBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.MessageId))
            {
                throw new ArgumentNullException("Не указан id сообщения", nameof(model.MessageId));
            }
            if (string.IsNullOrEmpty(model.SenderName))
            {
                throw new ArgumentNullException("Не указана почта отправителя", nameof(model.SenderName));
            }
            if (string.IsNullOrEmpty(model.Subject))
            {
                throw new ArgumentNullException("Не указан заголовок", nameof(model.Subject));
            }
            if (string.IsNullOrEmpty(model.Body))
            {
                throw new ArgumentNullException("Не указан текст сообщения", nameof(model.Subject));
            }
            _logger.LogInformation("MessageInfo. MessageId: {MessageId}. SenderName: {SenderName}. DateDelivery: {DateDelivery} Subject: {Subject}. Body: {Body}", model.MessageId, model.SenderName, model.DateDelivery, model.Subject, model.Body);
            var element = _clientStorage.GetElement(new ClientSearchModel
            {
                Email = model.SenderName
            });
            if (element == null)
            {
                _logger.LogWarning("Не удалось найти клиента, отправившего письмо с адреса Email: {Email}", model.SenderName);
            }
            else
            {
                model.ClientId = element.Id;
            }
        }
    }
}
