﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ConfectioneryContracts.BindingModels;
using ConfectioneryContracts.ViewModels;
using ConfectioneryDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Runtime.Serialization;

namespace ConfectioneryDatabaseImplement.Models
{
    [DataContract]
    public class Client : IClientModel
    {
        [DataMember]
        public int Id { get; private set; }
        [DataMember]
        [Required]
        public string ClientFIO { get; private set; } = string.Empty;
        [DataMember]
        [Required]
        public string Email { get; private set; } = string.Empty;
        [DataMember]
        [Required]
        public string Password { get; private set; } = string.Empty;
        [ForeignKey("ClientId")]
        public virtual List<Order> Orders { get; set; } = new();
        [ForeignKey("ClientId")]
        public virtual List<MessageInfo> ClientMessages { get; set; } = new();
        public static Client? Create(ClientBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new()
            {
                Id = model.Id,
                ClientFIO = model.ClientFIO,
                Email = model.Email,
                Password = model.Password
            };
        }
        public void Update(ClientBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            ClientFIO = model.ClientFIO;
            Email = model.Email;
            Password = model.Password;
        }
        public ClientViewModel GetViewModel => new()
        {
            Id = Id,
            ClientFIO = ClientFIO,
            Email = Email,
            Password = Password
        };
    }
}
