﻿using ConfectioneryContracts.BindingModels;
using ConfectioneryContracts.ViewModels;
using ConfectioneryDataModels.Enums;
using ConfectioneryDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.Runtime.Serialization;

namespace ConfectioneryDatabaseImplement.Models
{
    [DataContract]
    public class Order : IOrderModel
    {
        [DataMember]
        public int Id { get; set; }
        [DataMember]
        [Required]
        public int PastryId { get; set; }
        [DataMember]
        [Required]
        public int ClientId { get; set; }
        [DataMember]
        public int? ImplementerId { get; private set; }
        [DataMember]
        [Required]
        public int Count { get; set; }
        [DataMember]
        [Required]
        public double Sum { get; set; }
        [DataMember]
        [Required]
        public OrderStatus Status { get; set; }
        [DataMember]
        [Required]
        public DateTime DateCreate { get; set; }
        [DataMember]
        public DateTime? DateImplement { get; set; }
        public virtual Pastry Pastry { get; set; }
        public virtual Client Client { get; set; }
        public virtual Implementer? Implementer { get; set; }
        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                PastryId = model.PastryId,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            PastryId = PastryId,
            ClientId = ClientId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            PastryName = Pastry.PastryName,
            ClientFIO = Client.ClientFIO,
            ClientEmail = Client.Email ?? string.Empty,
            ImplementerFIO = Implementer?.ImplementerFIO ?? string.Empty
        };
    }
}