﻿using ConfectioneryContracts.BindingModels;
using ConfectioneryContracts.ViewModels;
using ConfectioneryDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace ConfectioneryFileImplement.Models
{
    [DataContract]
    public class MessageInfo : IMessageInfoModel
    {
        [DataMember]
        public int Id { get; private set; }
        [DataMember]
        public string MessageId { get; private set; } = string.Empty;
        [DataMember]
        public int? ClientId { get; private set; }
        [DataMember]
        public string SenderName { get; private set; } = string.Empty;
        [DataMember]
        public DateTime DateDelivery { get; private set; } = DateTime.Now;
        [DataMember]
        public string Subject { get; private set; } = string.Empty;
        [DataMember]
        public string Body { get; private set; } = string.Empty;

        public static MessageInfo? Create(MessageInfoBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new()
            {
                MessageId = model.MessageId,
                ClientId = model.ClientId,
                SenderName = model.SenderName,
                DateDelivery = model.DateDelivery,
                Subject = model.Subject,
                Body = model.Body
            };
        }

        public static MessageInfo? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new()
            {
                MessageId = element.Attribute("MessageId")!.Value,
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
                SenderName = element.Element("SenderName")!.Value,
                DateDelivery = Convert.ToDateTime(element.Element("DateDelivery")!.Value),
                Subject = element.Element("Subject")!.Value,
                Body = element.Element("Body")!.Value
            };
        }

        public MessageInfoViewModel GetViewModel => new()
        {
            MessageId = MessageId,
            ClientId = ClientId,
            SenderName = SenderName,
            DateDelivery = DateDelivery,
            Subject = Subject,
            Body = Body
        };

        public XElement GetXElement => new("MessageInfo",
            new XAttribute("MessageId", MessageId),
            new XElement("ClientId", ClientId),
            new XElement("SenderName", SenderName),
            new XElement("DateDelivery", DateDelivery),
            new XElement("Subject", Subject),
            new XElement("Body", Body)
            );
    }
}
