﻿using ConfectioneryContracts.BindingModels;
using ConfectioneryContracts.SearchModels;
using ConfectioneryContracts.StoragesContracts;
using ConfectioneryContracts.ViewModels;
using ConfectioneryListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ConfectioneryListImplement.Implements
{
    public class MessageInfoStorage : IMessageInfoStorage
    {
        private readonly DataListSingleton _source;

        public MessageInfoStorage()
        {
            _source = DataListSingleton.GetInstance();
        }

        public List<MessageInfoViewModel> GetFullList()
        {
            var result = new List<MessageInfoViewModel>();
            foreach (var message in _source.Messages)
            {
                result.Add(message.GetViewModel);
            }
            return result;
        }

        public List<MessageInfoViewModel> GetFilteredList(MessageInfoSearchModel model)
        {
            var result = new List<MessageInfoViewModel>();
            if (!model.ClientId.HasValue)
            {
                return result;
            }
            foreach (var message in _source.Messages)
            {
                if (message.ClientId.HasValue && message.ClientId == model.ClientId)
                {
                    result.Add(message.GetViewModel);
                }
            }
            return result;
        }

        public MessageInfoViewModel? GetElement(MessageInfoSearchModel model)
        {
            if (string.IsNullOrEmpty(model.MessageId))
            {
                return null;
            }
            foreach (var message in _source.Messages)
            {
                if (model.MessageId.Equals(message.MessageId))
                {
                    return message.GetViewModel;
                }
            }
            return null;
        }

        public MessageInfoViewModel? Insert(MessageInfoBindingModel model)
        {
            var newMessage = MessageInfo.Create(model);
            if (newMessage == null)
            {
                return null;
            }
            _source.Messages.Add(newMessage);
            return newMessage.GetViewModel;
        }
    }
}
